@echo off

:: Convert a CSV file to PDF form data files and merge them with a PDF form
:: CSV parse code from http://stackoverflow.com/a/6477991
::
:: Hans Sep November 12th 2015

:: Fiery JobFlow provided arguments
set INPUT=%~1
set OUTPUT=%~2
set JOBNAME=%~3
set WORKFLOW=%~4
set PREFLIGHT=%~5

::Some static variables
set ScriptName=%~n0
set BasePath=%~dp0
set LOG=%BasePath%\%ScriptName%.log
set SEVENZ="%BasePath%\7z\7za.exe"
set PDFTK="%BasePath%\pdftk\bin\pdftk.exe"

::Check if 7za is available
if not exist %SEVENZ% (
  echo %DATE% %TIME% [ERROR] Cannot find %SEVENZ%. Download 7z command line executable from http://7-zip.org. Exit script.>>%LOG%
  goto :EOF
)
::Check if pdftk is available
if not exist %PDFTK% (
  echo %DATE% %TIME% [ERROR] Cannot find %PDFTK%. Download PDF TK Server from https://www.pdflabs.com/tools/pdftk-server Exit script.>>%LOG%
  goto :EOF
)
:: Check if job is an archive
set EXT=%~x1
:: Note to self: JobFlow assumes archives need to be merged and renames the original archive.
if /I not "%EXT%"==".zip" (
  echo %DATE% %TIME% [ERROR] %INPUT% is not an archive. Script expect a zip archive. Script exit.>>%LOG%
  goto :EOF
)

::Some temporary locations we need to process PDF forms and PDF data
set TempBasePath=%BasePath%\%ScriptName%-tmp
if not exist %TempBasePath% mkdir %TempBasePath%
set WorkingPath=%TempBasePath%\tmp-%RANDOM%
if not exist %WorkingPath% mkdir %WorkingPath%

::Extract archive using 7z
echo %DATE% %TIME% [INFO] Unarchive via %7Z% e "%INPUT%" -y -o%WorkingPath%>>%LOG%
%SEVENZ% e "%INPUT%" -y -o%WorkingPath%

:: Check for PDFs, for now we only expect one.
for /f "delims=" %%o in ('attrib.exe %WorkingPath%\*.pdf ^| find /v "File not found - " ^| find /c /v ""') do set _c=%%o
if "%_c%"=="1" (
  for /f "tokens=*" %%f in ('dir /b %WorkingPath%\*.pdf') do (
    set PDFFORM=%WorkingPath%\%%f
    echo %DATE% %TIME% [INFO] PDF form found: %PDFFORM% >>%LOG%
  )
)
if %_c% geq 2 (
  echo %DATE% %TIME% [ERROR] Multiple PDFs found in %WorkingPath%. Script exit.>>%LOG%
  goto :EOF
)
if "%_c%"=="" (
  echo %DATE% %TIME% [ERROR] No PDF forms found. Script Exit>>%LOG%
  goto :EOF
)

:: Check for CSVs, we only expect one.
for /f "delims=" %%o in ('attrib.exe %WorkingPath%\*.csv ^| find /v "File not found - " ^| find /c /v ""') do set _c=%%o
if "%_c%"=="1" (
  for /f "tokens=*" %%f in ('dir /b %WorkingPath%\*.csv') do (
    set PDFDATA=%WorkingPath%\%%~f
    echo %DATE% %TIME% [INFO] Data file found: %PDFDATA% >>%LOG%
  )
)
if %_c% geq 2 (
  echo %DATE% %TIME% [ERROR] Multiple data files found in %WorkingPath%. Script exit.>>%LOG%
  goto :EOF
)
if "%_c%"=="" (
  echo %DATE% %TIME% [ERROR] No PDF data found. Script Exit>>%LOG%
  goto :EOF
)

:: Parse data file and create PDF form data files
for /F "usebackq tokens=1-10 delims=," %%a in ("%PDFDATA%") do call :process "%%a" "%%b" "%%c" "%%d" "%%e" "%%f" "%%g" "%%h" "%%i" "%%j"
:: Check if we need to merge output
call :catpdfs

goto :EOF

:trim
set "tmp=%~1"
:trimlead
if NOT "%tmp:~0,1%"==" " goto :EOF
set "tmp=%tmp:~1%"
goto trimlead

:process
set /A linenum+=1
if "%linenum%" geq "2" call :fdfheader
if "%linenum%"=="1" goto picknames
set ind=0

:display
if "%fcount%"=="%ind%" (
  call :fdffooter
  call :mergefdf
  goto :EOF
)
set /A ind+=1
call :trim %1
setLOCAL ENABLEDELAYEDEXPANSION
call :fdfvalues "!f%ind%!" "!tmp!"
ENDLOCAL
shift
goto display

:picknames
if %1=="" goto :EOF
CALL :trim %1
set /a fcount+=1
set "f%fcount%=%tmp%"
SHifT
goto picknames

:fdfheader
set RND=%RANDOM%
echo %%FDF-1.2 >>%WorkingPath%\data-%linenum%.fdf
echo 1 0 obj^<^</FDF^<^< /Fields[ >>%WorkingPath%\data-%linenum%.fdf
goto :EOF

:fdfvalues
echo ^<^</T(%~1)/V(%~2)^>^> >>%WorkingPath%\data-%linenum%.fdf
goto :EOF

:fdffooter
echo ] ^>^> ^>^>endobj >>%WorkingPath%\data-%linenum%.fdf
echo trailer >>%WorkingPath%\data-%linenum%.fdf
echo ^<^</Root 1 0 R^>^> >>%WorkingPath%\data-%linenum%.fdf
echo %%%%EOF >>%WorkingPath%\data-%linenum%.fdf
echo %DATE% %TIME% [INFO] Created FDF %WorkingPath%\data-%linenum%.fdf>>%LOG%
goto :EOF

:mergefdf
%PDFTK% %PDFFORM% fill_form %WorkingPath%\data-%linenum%.fdf output %WorkingPath%\pdf-%linenum%.pdf
echo %DATE% %TIME% [INFO] Created %WorkingPath%\pdf-%linenum%.pdf from %WorkingPath%\data-%linenum%.fdf>>%LOG%
goto :EOF

:catpdfs
:: Check for PDFs, if there are more than 1 merge them in to a single PDF.
for /f "delims=" %%o in ('attrib.exe %WorkingPath%\pdf-*.pdf ^| find /v "File not found - " ^| find /c /v ""') do set _c=%%o
if %_c% geq 2 (
  echo %DATE% %TIME% [INFO] Multiple PDFs found in %WorkingPath%. Start Merge.>>%LOG%
  %PDFTK% %WorkingPath%\pdf-*.pdf cat output %WorkingPath%\merged.pdf
  move /Y %WorkingPath%\merged.pdf %OUTPUT%
  echo %DATE% %TIME% [INFO] Moved %WorkingPath%\merged.pdf to %OUTPUT%.>>%LOG%
  goto :EOF
)
if "%_c%"=="1" (
  for /f "tokens=*" %%f in ('dir /b %WorkingPath%\pdf-*.pdf') do (
    move /Y %WorkingPath%\%%f %OUTPUT%
    echo %DATE% %TIME% [INFO] Moved %WorkingPath%\%%f to %OUTPUT%.>>%LOG%
  )
  goto :EOF
)
echo %DATE% %TIME% [ERROR] No PDFs found in %WorkingPath%. Script exit.>>%LOG%
goto :EOF